
import 'dart:async';
import 'dart:math';
import 'dart:math' as math;

import 'package:flutter/material.dart';

import 'enum/progressEnum.dart';
import 'enum/pullToRefreshMode.dart';
import 'indicater/circular_progress.dart';
import 'indicater/clipper.dart';
import 'refreshController.dart';

// The over-scroll distance that moves the indicator to its maximum
// displacement, as a percentage of the scrollable's container extent.
const double _kDragContainerExtentPercentage = 0.25;

// How much the scroll's drag gesture can overshoot the LiquidPullToRefresh's
// displacement; max displacement = _kDragSizeFactorLimit * displacement.
const double _kDragSizeFactorLimit = 1.5;

// When the scroll ends, the duration of the progress indicator's animation
// to the LiquidPullToRefresh's displacement.
// const Duration _kIndicatorSnapDuration = Duration(milliseconds: 150);

// The duration of the ScaleTransitionIn of box that starts when the
// refresh action has completed.
const Duration _kIndicatorScaleDuration = Duration(milliseconds: 200);

/// The signature for a function that's called when the user has dragged a
/// [PullToRefresh] far enough to demonstrate that they want the app to
/// refresh. The returned [Future] must complete when the refresh operation is
/// finished.
///
/// Used by [PullToRefresh.onRefresh].
typedef RefreshCallback = Future<void> Function();


class PullToRefresh extends StatefulWidget {
  const PullToRefresh({
    Key? key,
    this.animSpeedFactor = 1.0,
    required this.child,
    required this.onFail,
    required this.onComplete,
    required this.onLoading,
    required this.onRefresh,
    required this.refreshController,
    this.pullIndecator,
    this.backgroundColor,
    this.tColor,
    this.height,
    this.springAnimationDurationInMilliseconds = 1000,
    this.borderWidth = 2.0,
    this.showChildOpacityTransition = true,
  })  : assert(animSpeedFactor >= 1.0),
        super(key: key);

  /// The widget below this widget in the tree.
  ///
  /// The progress indicator will be stacked on top of this child. The indicator
  /// will appear when child's Scrollable descendant is over-scrolled.
  ///
  /// Typically a [ListView] or [CustomScrollView].
  final Widget child;

  final Widget? pullIndecator;

  /// The widget below this widget in the tree.
  ///
  /// The indicator will appear during loading.
  ///
  final Text onLoading;

  /// The widget below this widget in the tree.
  ///
  /// The indicator will appear on complete loading.
  ///
  final Widget onComplete;

  /// The widget below this widget in the tree.
  ///
  /// The indicator will appear on fail loading.
  ///
  final Widget onFail;

  /// The distance from the child's top or bottom edge to where the box
  /// will settle after the spring effect.
  ///
  /// default is set to 100.0
  final double? height;

  /// Duration in milliseconds of springy effect that occurs when
  /// we leave dragging after full drag.
  ///
  /// default to 1000
  final int springAnimationDurationInMilliseconds;

  /// To regulate the "speed of the animation" towards the end.
  /// To hasten it give a value > 1.0 and vice versa.
  ///
  /// default to 1.0
  final double animSpeedFactor;

  /// Border width of progressing circle in Progressing Indicator
  ///
  /// default to 2.0
  final double borderWidth;

  /// Whether to show child opacity transition or not.
  ///
  /// default to true
  final bool showChildOpacityTransition;

  /// A function that's called when the user has dragged the progress indicator
  /// far enough to demonstrate that they want the app to refresh. The returned
  /// [Future] must complete when the refresh operation is finished.
  // final RefreshCallback onRefresh;
  final VoidCallback? onRefresh;

  /// The progress indicator's foreground color. The current theme's
  /// [Theme.of(context).colorScheme.secondary] by default.
  final Color? backgroundColor;

  /// The progress indicator's background color. The current theme's
  /// [ThemeData.canvasColor] by default.
  final Color? tColor;

  final RefreshController refreshController;

  @override
  PullToRefreshState createState() => PullToRefreshState();
}

class PullToRefreshState extends State<PullToRefresh>
    with TickerProviderStateMixin<PullToRefresh> {

  late AnimationController _springController;
  late Animation<double> _springAnimation;

  late AnimationController _progressingController;
  late Animation<double> _progressingRotateAnimation;
  late Animation<double> _progressingPercentAnimation;
  late Animation<double> _progressingStartAngleAnimation;

  late AnimationController _ringDisappearController;
  late Animation<double> _ringRadiusAnimation;
  late Animation<double> _ringOpacityAnimation;

  late AnimationController _showPeakController;
  late Animation<double> _peakHeightUpAnimation;
  late Animation<double> _peakHeightDownAnimation;

  late AnimationController _indicatorMoveWithPeakController;
  late Animation<double> _indicatorTranslateWithPeakAnimation;
  late Animation<double> _indicatorRadiusWithPeakAnimation;

  late AnimationController _indicatorTranslateInOutController;
  late Animation<double> _indicatorTranslateAnimation;

  late AnimationController _radiusController;
  late Animation<double> _radiusAnimation;

  late Animation<double> _childOpacityAnimation;

  late AnimationController _positionController;
  late Animation<double> _value;
  late Animation<Color?> _valueColor;

  PullToRefreshMode? _mode;
  Future<void>? _pendingRefreshFuture;
  bool? _isIndicatorAtTop;
  double? _dragOffset;

  /// show widget on complete
  ProgressStatus _progressStatus = ProgressStatus.loading;

  Widget? indicaterWidget;

  static final Animatable<double> _threeQuarterTween =
      Tween<double>(begin: 0.0, end: 0.75);
  static final Animatable<double> _oneToZeroTween =
      Tween<double>(begin: 1.0, end: 0.0);

  @override
  void initState() {
    super.initState();
    _springController = AnimationController(vsync: this);
    _springAnimation =
        _springController.drive(Tween<double>(begin: 1.0, end: -1.0));

    _progressingController = AnimationController(
        vsync: this, duration: const Duration(milliseconds: 1000));
    _progressingRotateAnimation =
        Tween<double>(begin: 0.0, end: 1.0).animate(CurvedAnimation(
      parent: _progressingController,
      curve: const Interval(0.0, 1.0),
    ));
    _progressingPercentAnimation =
        Tween<double>(begin: 0.25, end: 5 / 6).animate(CurvedAnimation(
      parent: _progressingController,
      curve: Interval(0.0, 1.0, curve: ProgressRingCurve()),
    ));
    _progressingStartAngleAnimation =
        Tween<double>(begin: -2 / 3, end: 1 / 2).animate(CurvedAnimation(
      parent: _progressingController,
      curve: const Interval(0.5, 1.0),
    ));

    _ringDisappearController = AnimationController(vsync: this);
    _ringRadiusAnimation = Tween<double>(begin: 1.0, end: 1.25).animate(
        CurvedAnimation(
            parent: _ringDisappearController,
            curve: const Interval(0.0, 0.2, curve: Curves.easeOut)));
    _ringOpacityAnimation = Tween<double>(begin: 1.0, end: 0.0).animate(
        CurvedAnimation(
            parent: _ringDisappearController,
            curve: const Interval(0.0, 0.2, curve: Curves.easeIn)));

    _showPeakController = AnimationController(vsync: this);
    _peakHeightUpAnimation = Tween<double>(begin: 0.0, end: 1.0).animate(
        CurvedAnimation(
            parent: _showPeakController,
            curve: const Interval(0.1, 0.2, curve: Curves.easeOut)));
    _peakHeightDownAnimation = Tween<double>(begin: 1.0, end: 0.0).animate(
        CurvedAnimation(
            parent: _showPeakController,
            curve: const Interval(0.2, 0.3, curve: Curves.easeIn)));

    _indicatorMoveWithPeakController = AnimationController(vsync: this);
    _indicatorTranslateWithPeakAnimation = Tween<double>(begin: 0.0, end: 1.0)
        .animate(CurvedAnimation(
            parent: _indicatorMoveWithPeakController,
            curve: const Interval(0.1, 0.2, curve: Curves.easeOut)));
    _indicatorRadiusWithPeakAnimation = Tween<double>(begin: 0.0, end: 1.0)
        .animate(CurvedAnimation(
            parent: _indicatorMoveWithPeakController,
            curve: const Interval(0.1, 0.2, curve: Curves.easeOut)));

    _indicatorTranslateInOutController = AnimationController(vsync: this);
    _indicatorTranslateAnimation = Tween<double>(begin: 0.0, end: 1.0).animate(
        CurvedAnimation(
            parent: _indicatorTranslateInOutController,
            curve: const Interval(0.2, 0.6, curve: Curves.easeOut)));

    _radiusController = AnimationController(vsync: this);
    _radiusAnimation = Tween<double>(begin: 0.0, end: 1.0).animate(
        CurvedAnimation(parent: _radiusController, curve: Curves.easeIn));

    _positionController = AnimationController(vsync: this);
    _value = _positionController.drive(_threeQuarterTween);

    _childOpacityAnimation = _positionController.drive(_oneToZeroTween);

    if (widget.refreshController != null) {
      widget.refreshController.setFRefreshState(this);
    }
  }

  @override
  void didChangeDependencies() {
    final ThemeData theme = Theme.of(context);
    _valueColor = _positionController.drive(
      ColorTween(
              begin: (widget.backgroundColor ?? theme.colorScheme.secondary)
                  .withOpacity(0.0),
              end: (widget.backgroundColor ?? theme.colorScheme.secondary)
                  .withOpacity(1.0))
          .chain(CurveTween(
              curve: const Interval(0.0, 1.0 / _kDragSizeFactorLimit))),
    );
    super.didChangeDependencies();
  }

  @override
  void dispose() {
    _springController.dispose();
    _progressingController.dispose();
    _positionController.dispose();
    _ringDisappearController.dispose();
    _showPeakController.dispose();
    _indicatorMoveWithPeakController.dispose();
    _indicatorTranslateInOutController.dispose();
    _radiusController.dispose();
    super.dispose();
  }

  bool _handleScrollNotification(ScrollNotification notification) {
    if (notification is ScrollStartNotification &&
        notification.metrics.extentBefore == 0.0 &&
        _mode == null &&
        _start(notification.metrics.axisDirection)) {
      setState(() {
        _mode = PullToRefreshMode.drag;
      });
      return false;
    }
    bool? indicatorAtTopNow;
    switch (notification.metrics.axisDirection) {
      case AxisDirection.down:
        indicatorAtTopNow = true;
        break;
      case AxisDirection.up:
        indicatorAtTopNow = false;
        break;
      case AxisDirection.left:
      case AxisDirection.right:
        indicatorAtTopNow = null;
        break;
    }
    if (indicatorAtTopNow != _isIndicatorAtTop) {
      if (_mode == PullToRefreshMode.drag ||
          _mode == PullToRefreshMode.armed) {
        _dismiss(PullToRefreshMode.canceled);
      }
    } else if (notification is ScrollUpdateNotification) {
      if (_mode == PullToRefreshMode.drag ||
          _mode == PullToRefreshMode.armed) {
        if (notification.metrics.extentBefore > 0.0) {
          _dismiss(PullToRefreshMode.canceled);
        } else {
          if (_dragOffset != null) {
            _dragOffset = _dragOffset! - notification.scrollDelta!;
          }
          _checkDragOffset(notification.metrics.viewportDimension);
        }
      }
      if (_mode == PullToRefreshMode.armed &&
          notification.dragDetails == null) {
        // On iOS start the refresh when the Scrollable bounces back from the
        // OverScroll (ScrollNotification indicating this don't have dragDetails
        // because the scroll activity is not directly triggered by a drag).
        _show();
      }
    } else if (notification is OverscrollNotification) {
      if (_mode == PullToRefreshMode.drag ||
          _mode == PullToRefreshMode.armed) {
        if (_dragOffset != null) {
          _dragOffset = _dragOffset! - notification.overscroll / 2.0;
        }
        _checkDragOffset(notification.metrics.viewportDimension);
      }
    } else if (notification is ScrollEndNotification) {
      switch (_mode) {
        case PullToRefreshMode.armed:
          _show();
          break;
        case PullToRefreshMode.drag:
          _dismiss(PullToRefreshMode.canceled);
          break;
        default:
          // do nothing
          break;
      }
    }
    return false;
  }

  bool _handleGlowNotification(OverscrollIndicatorNotification notification) {
    if (notification.depth != 0 || !notification.leading) return false;
    if (_mode == PullToRefreshMode.drag) {
      notification.disallowIndicator();
      return true;
    }
    return false;
  }

  // Stop showing the progress indicator.
  Future<void> _dismiss(PullToRefreshMode newMode) async {
    await Future<void>.value();
    // This can only be called from _show() when refreshing and
    // _handleScrollNotification in response to a ScrollEndNotification or
    // direction change.
    assert(newMode == PullToRefreshMode.canceled ||
        newMode == PullToRefreshMode.done);
    setState(() {
      _mode = newMode;
    });
    switch (_mode) {
      case PullToRefreshMode.done:
        //stop progressing animation
        _progressingController.stop();

        // progress ring disappear animation
        _ringDisappearController.animateTo(1.0,
            duration: Duration(
                milliseconds: (widget.springAnimationDurationInMilliseconds /
                        widget.animSpeedFactor)
                    .round()),
            curve: Curves.linear);

        // indicator translate out
        _indicatorMoveWithPeakController.animateTo(0.0,
            duration: Duration(
                milliseconds: (widget.springAnimationDurationInMilliseconds /
                        widget.animSpeedFactor)
                    .round()),
            curve: Curves.linear);
        _indicatorTranslateInOutController.animateTo(0.0,
            duration: Duration(
                milliseconds: (widget.springAnimationDurationInMilliseconds /
                        widget.animSpeedFactor)
                    .round()),
            curve: Curves.linear);

        //initial value of controller is 1.0 (indicator will move from top to bottom
        // await _showPeakController.animateTo(0.3,
        //     duration: Duration(
        //         milliseconds: (widget.springAnimationDurationInMilliseconds /
        //                 widget.animSpeedFactor)
        //             .round()),
        //     curve: Curves.linear);

        _radiusController.animateTo(0.0,
            duration: Duration(
                milliseconds: (widget.springAnimationDurationInMilliseconds /
                        (widget.animSpeedFactor * 5))
                    .round()),
            curve: Curves.linear);

        // _showPeakController.value = 0.175;
        // await _showPeakController.animateTo(0.1,
        //     duration: Duration(
        //         milliseconds: (widget.springAnimationDurationInMilliseconds /
        //                 (widget.animSpeedFactor * 5))
        //             .round()),
        //     curve: Curves.easeOut);
        _showPeakController.value = 0.0;

        await _positionController.animateTo(0.0,
            duration: Duration(
                milliseconds: (widget.springAnimationDurationInMilliseconds /
                        widget.animSpeedFactor)
                    .round()));
        break;

      case PullToRefreshMode.canceled:
        await _positionController.animateTo(0.0,
            duration: _kIndicatorScaleDuration);
        break;
      default:
        assert(false);
    }
    if (mounted && _mode == newMode) {
      _dragOffset = null;
      _isIndicatorAtTop = null;
      setState(() {
        _mode = null;
      });
    }
  }

  bool _start(AxisDirection direction) {
    assert(_mode == null);
    assert(_isIndicatorAtTop == null);
    assert(_dragOffset == null);
    switch (direction) {
      case AxisDirection.down:
        _isIndicatorAtTop = true;
        break;
      case AxisDirection.up:
        _isIndicatorAtTop = false;
        break;
      case AxisDirection.left:
      case AxisDirection.right:
        _isIndicatorAtTop = null;
        // we do not support horizontal scroll views.
        return false;
    }
    _dragOffset = 0.0;
    _positionController.value = 0.0;
    _springController.value = 0.0;
    _progressingController.value = 0.0;
    _ringDisappearController.value = 1.0;
    _showPeakController.value = 0.0;
    _indicatorMoveWithPeakController.value = 0.0;
    _indicatorTranslateInOutController.value = 0.0;
    _radiusController.value = 1.0;
    return true;
  }

  void _checkDragOffset(double containerExtent) {
    assert(_mode == PullToRefreshMode.drag ||
        _mode == PullToRefreshMode.armed);
    double newValue =
        _dragOffset! / (containerExtent * _kDragContainerExtentPercentage);
    if (_mode == PullToRefreshMode.armed) {
      newValue = math.max(newValue, 1.0 / _kDragSizeFactorLimit);
    }
    _positionController.value =
        newValue.clamp(0.0, 1.0); // this triggers various rebuilds
    if (_mode == PullToRefreshMode.drag &&
        _valueColor.value!.alpha == 0xFF) {
      _mode = PullToRefreshMode.armed;
    }
  }

  void _show() {
    _progressStatus = ProgressStatus.loading;
    indicaterWidget = widget.onLoading;
    assert(_mode != PullToRefreshMode.refresh);
    assert(_mode != PullToRefreshMode.snap);
    final Completer<void> completer = Completer<void>();
    _pendingRefreshFuture = completer.future;
    _mode = PullToRefreshMode.snap;

    _positionController.animateTo(1.0 / _kDragSizeFactorLimit,
        duration: Duration(
            milliseconds: widget.springAnimationDurationInMilliseconds),
        curve: Curves.linear);

    _showPeakController.animateTo(1.0,
        duration: Duration(
            milliseconds: widget.springAnimationDurationInMilliseconds),
        curve: Curves.linear);

    //indicator translate in with peak
    _indicatorMoveWithPeakController.animateTo(1.0,
        duration: Duration(
            milliseconds: widget.springAnimationDurationInMilliseconds),
        curve: Curves.linear);

    //indicator move to center
    _indicatorTranslateInOutController.animateTo(1.0,
        duration: Duration(
            milliseconds: widget.springAnimationDurationInMilliseconds),
        curve: Curves.linear);

    // progress ring fade in
    _ringDisappearController.animateTo(0.0,
        duration: Duration(
            milliseconds: widget.springAnimationDurationInMilliseconds));

    _springController
        .animateTo(0.5,
            duration: Duration(
                milliseconds: widget.springAnimationDurationInMilliseconds),
            curve: Curves.elasticOut)
        .then<void>((void value) {
      if (mounted && _mode == PullToRefreshMode.snap) {
        setState(() {
          // Show the indeterminate progress indicator.
          _mode = PullToRefreshMode.refresh;
        });

        //run progress animation
        _progressingController.repeat();
        if(widget.onRefresh != null){
          widget.onRefresh!();
        }
      }
    });
  }

  /// this function will call on succeed process
  Future<void> onComplete() async {
    if (mounted && _mode == PullToRefreshMode.refresh) {
      _progressStatus = ProgressStatus.completed;
      indicaterWidget = widget.onComplete;
      await Future.delayed(const Duration(milliseconds:500));
      //completer.complete();

      _dismiss(PullToRefreshMode.done);
    }
  }

  /// this function will call for failed process
  Future<void> onFailed() async {
    if (mounted && _mode == PullToRefreshMode.refresh) {
      _progressStatus = ProgressStatus.failed;
      indicaterWidget = widget.onFail;
      await Future.delayed(const Duration(milliseconds:500));
      //completer.complete();

      _dismiss(PullToRefreshMode.done);
    }
  }

  /// Show the progress indicator and run the refresh callback as if it had
  /// been started interactively. If this method is called while the refresh
  /// callback is running, it quietly does nothing.
  ///
  /// Creating the [PullToRefresh] with a [GlobalKey<LiquidPullToRefreshState>]
  /// makes it possible to refer to the [PullToRefreshState].
  ///
  /// The future returned from this method completes when the
  /// [PullToRefresh.onRefresh] callback's future completes.
  ///
  /// If you await the future returned by this function from a [State], you
  /// should check that the state is still [mounted] before calling [setState].
  ///
  /// When initiated in this manner, the progress indicator is independent of any
  /// actual scroll view. It defaults to showing the indicator at the top. To
  /// show it at the bottom, set `atTop` to false.
  Future<void>? show({bool atTop = true}) {
    if (_mode != PullToRefreshMode.refresh &&
        _mode != PullToRefreshMode.snap) {
      if (_mode == null) _start(atTop ? AxisDirection.down : AxisDirection.up);
      _show();
    }
    return _pendingRefreshFuture;
  }

  final GlobalKey _key = GlobalKey();

  @override
  Widget build(BuildContext context) {
    assert(debugCheckHasMaterialLocalizations(context));

    // assigning default color and background color
    Color _defaultColor = Theme.of(context).colorScheme.secondary;
    Color _defaultBackgroundColor = Theme.of(context).canvasColor;

    // assigning default height
    double _defaultHeight = 100.0;

    // checking whether to take default values or not
    Color color = (widget.backgroundColor != null) ? widget.backgroundColor! : _defaultColor;
    Color backgroundColor = (widget.tColor != null)
        ? widget.tColor!
        : _defaultBackgroundColor;
    double height = (widget.height != null) ? widget.height! : _defaultHeight;

    final Widget child = NotificationListener<ScrollNotification>(
      key: _key,
      onNotification: _handleScrollNotification,
      child: NotificationListener<OverscrollIndicatorNotification>(
          onNotification: _handleGlowNotification, child: widget.child),
    );

    if (_mode == null) {
      assert(_dragOffset == null);
      assert(_isIndicatorAtTop == null);
      return child;
    }
    assert(_dragOffset != null);
    assert(_isIndicatorAtTop != null);

    return Stack(
      children: <Widget>[
        AnimatedBuilder(
          animation: _positionController,
          child: child,
          builder: (BuildContext buildContext, Widget? child) {
            if (widget.showChildOpacityTransition) {
              return Opacity(
                  // -0.01 is done for elasticOut curve
                  opacity: (widget.showChildOpacityTransition)
                      ? (_childOpacityAnimation.value - (1 / 3) - 0.01)
                          .clamp(0.0, 1.0)
                      : 1.0,
                  child: child);
            }
            return Transform.translate(
              offset: Offset(0.0, _positionController.value * height * 1.5),
              child: child,
            );
          },
        ),
        AnimatedBuilder(
          animation: Listenable.merge([
            _positionController,
            _springController,
            _showPeakController,
          ]),
          builder: (BuildContext buildContext, Widget? child) {
            return ClipPath(
              clipper: CurveHillClipper(
                centreHeight: height,
                curveHeight: height / 2 * _springAnimation.value, // 50.0
                peakHeight: height *
                    3 /
                    10 *
                    ((_peakHeightUpAnimation.value != 1.0) //30.0
                        ? _peakHeightUpAnimation.value
                        : _peakHeightDownAnimation.value),
                peakWidth: (_peakHeightUpAnimation.value != 0.0 &&
                        _peakHeightDownAnimation.value != 0.0)
                    ? height * 35 / 100 //35.0
                    : 0.0,
              ),
              child: Container(
                height: _value.value * height * 2, // 100.0
                color: color,
                alignment: Alignment.bottomCenter,
                child:_mode == PullToRefreshMode.drag
                    ? widget.pullIndecator ?? Row(
                      crossAxisAlignment: CrossAxisAlignment.center,
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: <Widget>[
                        Icon(
                          Icons.arrow_downward,
                          color: Colors.grey,
                        ),
                      ],
                    )
                    :const SizedBox()
              ),
            );
          },
        ),
        Container(
          height: height, //100.0
          child: AnimatedBuilder(
            animation: Listenable.merge([
              _progressingController,
              _ringDisappearController,
              _indicatorMoveWithPeakController,
              _indicatorTranslateInOutController,
              _radiusController,
            ]),
            builder: (BuildContext buildContext, Widget? child) {
              return Align(
                alignment: Alignment(
                  0.0,
                  (1.0 -
                      (0.36 * _indicatorTranslateWithPeakAnimation.value) -
                      (0.64 * _indicatorTranslateAnimation.value)),
                ),
                child:
                _progressStatus == ProgressStatus.loading?
                  Row(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    mainAxisAlignment: MainAxisAlignment.center,
                    children:[
                      Transform(
                        transform: Matrix4.identity()
                          ..rotateZ(_progressingRotateAnimation.value * 5 * pi / 6),
                        alignment: FractionalOffset.center,
                        child: CircularProgress(
                          backgroundColor: backgroundColor,
                          progressCircleOpacity: _ringOpacityAnimation.value,
                          innerCircleRadius: height *
                              15 /
                              100 * // 15.0
                              ((_mode != PullToRefreshMode.done)
                                  ? _indicatorRadiusWithPeakAnimation.value
                                  : _radiusAnimation.value),
                          progressCircleBorderWidth: widget.borderWidth,
                          //2.0
                          progressCircleRadius: (_ringOpacityAnimation.value != 0.0)
                              ? (height * 2 / 10) * _ringRadiusAnimation.value //20.0
                              : 0.0,
                          startAngle: _progressingStartAngleAnimation.value * pi,
                          progressPercent: _progressingPercentAnimation.value,
                        ),
                      ),
                      SizedBox(width: 5,),
                      indicaterWidget != null ?
                        Opacity(
                            opacity:  getIndicaterOpcity(height) > 0 ? 1: 0,
                             child:indicaterWidget ?? const Text("Loading..")
                         )
                        :const SizedBox()

                    ],
                  )
                  :_progressStatus == ProgressStatus.completed?
                    Opacity(
                      opacity: getIndicaterOpcity(height) > 0 ? 1: 0,
                      child: Row(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        mainAxisAlignment: MainAxisAlignment.center,
                        children:[
                          indicaterWidget ?? const Text("Succeed")

                        ],
                      ),
                    )
                    :Opacity(
                      opacity: getIndicaterOpcity(height) > 0 ? 1: 0,
                      child:Row(
                        crossAxisAlignment: CrossAxisAlignment.center,
                        mainAxisAlignment: MainAxisAlignment.center,
                        children:[
                          indicaterWidget ?? const Text("Failed")

                        ],
                      )
                    )
              );
            },
          ),
        ),
      ],
    );
  }

  getIndicaterOpcity(double height){
   double innerCircleRadius = height *
        15 /
        100 * // 15.0
        ((_mode != PullToRefreshMode.done)
            ? _indicatorRadiusWithPeakAnimation.value
            : _radiusAnimation.value);

   double progressCircleRadius= (_ringOpacityAnimation.value != 0.0)
       ? (height * 2 / 10) * _ringRadiusAnimation.value //20.0
       : 0.0;

   double containerOpcity =  2 * max(progressCircleRadius, innerCircleRadius);

   return containerOpcity;

  }

}


class ProgressRingCurve extends Curve {
  @override
  double transform(double t) {
    if (t <= 0.5) {
      return 2 * t;
    } else {
      return 2 * (1 - t);
    }
  }
}
